function rp_mid_u_with_bas_mesh(nn, K, n, isfig4)
paraminput = paramset();
u_hat = 1.4;
x_i = log(2)/u_hat;
c = 10;
PIN = 0.35;
fixBAS = 0.03;

b_vec = linspace(0.01, 0.9, nn)';
sigma_u_vec = linspace(0.01, 0.8, nn)';
paraminput.a = 1*c;

[b_array, sigma_u_array] = meshgrid(b_vec, sigma_u_vec);
rp_array = nan(size(b_array));
rp_no_unc_array = nan(size(b_array));
RD_array = nan(size(b_array));
bas_array = nan(size(b_array));

for i = 1:numel(b_array)
    paraminput.b = b_array(i);
    unc_measure = sigma_u_array(i);
    [rp_array(i), ~, V_i] = rp_mid_u(c, paraminput, K, n, u_hat, x_i, unc_measure);
    lambda = exp(u_hat*x_i);
    [~, rp_no_unc_array(i)] = rp_mid(c, paraminput, K, n, lambda, unc_measure);
    bas_array(i) = fixBAS + PIN * abs(Vn_fun(c, paraminput, K, n, exp((u_hat + unc_measure)*x_i) ) - Vn_fun(c, paraminput, K, n, exp((u_hat - unc_measure)*x_i) ) )...
        /((Vn_fun(c, paraminput, K, n, exp((u_hat + unc_measure)*x_i) ) + Vn_fun(c, paraminput, K, n, exp((u_hat - unc_measure)*x_i) ))/2);
    RD_array(i) = (paraminput.a + paraminput.b*c)/abs(V_i);
end

if isfig4

    if K == 5 && n == 2
        set(gcf, 'Position', [10 10 1250 900]*0.6)
        set(gcf, 'Color', 'w');
        plot(sigma_u_array(:, end), bas_array(:,6)*100, 'LineWidth', 1.3)
        xlabel('$\sigma_{\hat{u}}$', 'Interpreter', 'latex')
        ylabel('Bid-ask spreads (percentage)', 'Interpreter', 'latex')
        title(['$\hat{u} = ' num2str(u_hat) ', \widehat{\lambda} = ' num2str(lambda) ', x = ' num2str(x_i,2) ', \alpha = ' num2str(PIN*100) '\%, \kappa = ' num2str(fixBAS*100) '\% $'],'Interpreter', 'latex')
        box off
        grid minor
    end

else


    figure
    set(gcf, 'Position', [10 10 2250 1050]*0.55)
    set(gcf, 'Color', 'w');
    subplot(1,2,1)
    hold on
    mesh(abs(RD_array), bas_array, abs(rp_array)*100, 'FaceColor', 'flat')
    mesh(abs(RD_array), bas_array, abs(rp_no_unc_array)*100, 'FaceColor', [0.8,0.8,0.8], 'EdgeColor', 'none')
    hold off
    legend({'Risk premium with uncertainty', 'Risk premium without uncertainty'}, 'Location', 'west', 'Interpreter', 'latex', 'Box', 'off')
    xlabel('RDME', 'Interpreter', 'latex')
    ylabel('Bid-ask spread', 'Interpreter', 'latex')
    zlabel('R\&D risk premium (percent)', 'Interpreter', 'latex')
    title(['$\hat{u} = ' num2str(u_hat) ', \widehat{\lambda} = ' num2str(exp(u_hat*x_i)) ', x = ' num2str(x_i,2) '$'],'Interpreter', 'latex')
    view([-41, 17])
    set(gca, 'Projection','perspective', 'Box', 'off')
    alpha 0.5
    grid minor

    subplot(1,2,2)
    rp_slope = (abs(rp_array(:,end)) - abs(rp_array(:,1)))./(abs(RD_array(:,end)) - abs(RD_array(:,1)))*1e+4;
    bas4plot = linspace(bas_array(1,1), bas_array(end,1), nn*10);
    rp_slope4plot = interp1(bas_array(:,1) , rp_slope, bas4plot, 'spline', 'extrap');
    plot(bas4plot, rp_slope4plot);
    xlabel('Bid-ask spread', 'Interpreter', 'latex')
    ylabel('R\&D risk premium slope (basis points)', 'Interpreter', 'latex')
    title(['$\hat{u} = ' num2str(u_hat) ', \widehat{\lambda} = ' num2str(exp(u_hat*x_i)) ', x = ' num2str(x_i,2) '$'],'Interpreter', 'latex')
    box off
    grid minor

    sgtitle(['Stage $n = ' num2str(n) '$'], 'Interpreter', 'latex')

end