function [ results ] = factorreg( pret,method )
% The function is used for running time series factor models. The input
% 'pret' is a metrix of portfolio returns which contains several
% portfolios. The 'method' represents number of factors. Note: before using
% the function, we should define the factor ourself, for example, we need to
% define the mkt is factor1 and go forth.
%-------------------------------- Set Up ---------------------------------%
load factors
load dates
i  = find(ffdates ==  dates(1));                            % match the dates
t  = find(ffdates == dates(end));

if method == 1;                                             % construct independent variables
    x  = [ones(size(dates)) factor1(i:t)];
elseif method == 2;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t)];
elseif method == 3;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t)] ;
elseif method == 4;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t) factor4(i:t)];
elseif method == 5;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t) factor4(i:t) factor5(i:t)];
end
rf = rf(i:t);
%------------------------ Portfolio Regression ---------------------------%
n = cols(pret);                                             % number of portfolios
for j = 1:n;
    y = pret(:,j) - rf;                                     % excess return
    res = nanols(y,x);
    
    alpha(j) = res.beta(1)*100;                             % common elements for all models
    talpha(j) = res.tstat(1);
    sharpe(j) = sqrt(12)*nanmean(y)/nanstd(y);
    info(j) = sqrt(12)*alpha(j)/(100*sqrt(res.sige));
    R2(j) = res.rsqr;
    b_factor1(j) = res.beta(2);
    t_factor1(j) = res.tstat(2);
      
    if method == 2 | method == 3 | method == 4 | method == 5;
        b_factor2(j) = res.beta(3);
        t_factor2(j) = res.tstat(3);
    end
    
    if method == 3 | method == 4 | method == 5;
        b_factor3(j) = res.beta(4);
        t_factor3(j) = res.tstat(4);
    end
    
    if method == 4 | method == 5;
        b_factor4(j) = res.beta(5);
        t_factor4(j) = res.tstat(5);
    end
    
    if method == 5;
        b_factor5(j) = res.beta(6);
        t_factor5(j) = res.tstat(6);
    end
    
    resx = nanols(y,ones(size(y)));                         % calculate excess portfolio returns
    b_xret(j) = resx.beta*100;
    t_xret(j) = resx.tstat;
end
%------------------- Long-Short Portfolio Regression ---------------------%
LSret  = pret(:,end) - pret(:,1);                           % long-short portfolio returns
yls = LSret;

resls = nanols(yls,x);
R2_ls = resls.rsqr;
alpha_ls = resls.beta(1)*100;
talpha_ls = resls.tstat(1);  
sharpe_ls = sqrt(12)*nanmean(yls)/nanstd(yls);
info_ls = sqrt(12)*alpha_ls/(100*sqrt(resls.sige));

b_factor1_ls = resls.beta(2);
t_factor1_ls = resls.tstat(2);

if method == 2 | method == 3 | method == 4 | method == 5;
    b_factor2_ls = resls.beta(3);
    t_factor2_ls = resls.tstat(3);
end

if method == 3 | method == 4 | method == 5;
    b_factor3_ls = resls.beta(4);
    t_factor3_ls = resls.tstat(4);
end

if method == 4 | method == 5;
    b_factor4_ls = resls.beta(5);
    t_factor4_ls = resls.tstat(5);
end
    
if method == 5;
    b_factor5_ls = resls.beta(6);
    t_factor5_ls = resls.tstat(6);
end

resx = nanols(yls,ones(size(yls)));
b_xret_ls = resx.beta*100;                                  % average excess long-short portfolio return
t_xret_ls = resx.tstat;
%---------------------------- Store Results ------------------------------%
a = [alpha alpha_ls];
t = [talpha talpha_ls];
sharpe = [sharpe sharpe_ls];
info = [info info_ls];
b_factor1 = [b_factor1 b_factor1_ls];
t_factor1 = [t_factor1 t_factor1_ls];

if method == 2 | method == 3 | method ==4 | method ==5;
    b_factor2 = [b_factor2 b_factor2_ls];
    t_factor2 = [t_factor2 t_factor2_ls];
end;

if method == 3 | method ==4 | method ==5;
    b_factor3 = [b_factor3 b_factor3_ls];
    t_factor3 = [t_factor3 t_factor3_ls];
end

if method ==4 | method ==5;
    b_factor4 = [b_factor4 b_factor4_ls];
    t_factor4 = [t_factor4 t_factor4_ls];
end

if  method == 5;
    b_factor5 = [b_factor5 b_factor5_ls];
    t_factor5 = [t_factor5 t_factor5_ls];
end

b_xret = [b_xret b_xret_ls];
t_xret = [t_xret t_xret_ls];
r = [R2 R2_ls];

results.meth = '1 Factor';
results.pret = pret;
results.r2 = r;
results.sharpe = sharpe;
results.info = info;
results.alpha = a;
results.talpha = t;
results.xret = b_xret;
results.txret = t_xret;
results.factor1 = b_factor1;
results.tfactor1 = t_factor1;

if method == 2 | method == 3 | method == 4 | method == 5;
    results.meth = '2 Factors';
    results.factor2 = b_factor2;
    results.tfactor2 = t_factor2;
end

if method == 3 | method == 4 | method == 5;
    results.meth = '3 Factors';
    results.factor3 = b_factor3;
    results.tfactor3 = t_factor3;
end

if method == 4 | method == 5;
    results.meth = '4 Factors';
    results.factor4 = b_factor4;
    results.tfactor4 = t_factor4;
end

if method == 5;
    results.meth = '5 Factors';
    results.factor5 = b_factor5;
    results.tfactor5 = t_factor5;
end

end

