function [ results ] = factorregnw( pret,method )
% The function is used for running time series factor models. The input
% 'pret' is a metrix of portfolio returns which contains several
% portfolios. The 'method' represents number of factors. Note: before using
% the function, we should define the factor ourself, for example, we need to
% define the mkt is factor1 and go forth.
%-------------------------------- Set Up ---------------------------------%
load factors
load datestemp
i  = find(ffdates ==  dates(1));                            % match the dates
t  = find(ffdates == dates(end));

if method == 1;                                             % construct independent variables
    x  = [ones(size(dates)) factor1(i:t)];
elseif method == 2;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t)];
elseif method == 3;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t)] ;
elseif method == 4;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t) factor4(i:t)];
elseif method == 5;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t) factor4(i:t) factor5(i:t)];
elseif method == 6;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t) factor4(i:t) factor5(i:t) factor6(i:t)];
elseif method == 7;
    x  = [ones(size(dates)) factor1(i:t) factor2(i:t) factor3(i:t) factor4(i:t) factor5(i:t) factor6(i:t) factor7(i:t)];
end
rf = rf(i:t);
%------------------------ Portfolio Regression ---------------------------%
n = cols(pret);                                             % number of portfolios
for j = 1:n;
    y = pret(:,j) - rf;                                     % excess return
    res = nanols(y,x);
    
    alpha(j) = res.beta(1)*100;                             % common elements for all models
    talpha(j) = res.tstat(1);
    nwt = olsnwt(x,y);                                      % newey-west t-value for portfolio excess returns, note: the last line in the fuction 'olsnwt'
    nwtalpha(j) = nwt(1);                                   % contains''intercept',false,', that mean the hac fuction excludes an intercept since there is one in matrix 'x'
    residual(:,j) = res.resid;
    bstd(:,j) = res.bstd;
    sharpe(j) = sqrt(12)*nanmean(y)/nanstd(y);
    info(j) = sqrt(12)*alpha(j)/(100*sqrt(res.sige));
    R2(j) = res.rsqr;
    b_factor1(j) = res.beta(2);
    t_factor1(j) = res.tstat(2);
    nwt_factor1(j) = nwt(2);
    
    if method == 2 | method == 3 | method == 4 | method == 5| method == 6 | method == 7;
        b_factor2(j) = res.beta(3);
        t_factor2(j) = res.tstat(3);
        nwt_factor2(j) = nwt(3);
    end
    
    if method == 3 | method == 4 | method == 5| method == 6 | method == 7;
        b_factor3(j) = res.beta(4);
        t_factor3(j) = res.tstat(4);
        nwt_factor3(j) = nwt(4);
    end
    
      
    if method == 4 | method == 5 | method == 6 | method == 7;
        b_factor4(j) = res.beta(5);
        t_factor4(j) = res.tstat(5);
        nwt_factor4(j) = nwt(5);
    end
    
    if method == 5 | method == 6 | method == 7;
        b_factor5(j) = res.beta(6);
        t_factor5(j) = res.tstat(6);
        nwt_factor5(j) = nwt(6);
    end
    
    if method == 6 | method == 7;
        b_factor6(j) = res.beta(7);
        t_factor6(j) = res.tstat(7);
        nwt_factor6(j) = nwt(7);
    end
    
    if method == 7;
        b_factor7(j) = res.beta(8);
        t_factor7(j) = res.tstat(8);
        nwt_factor7(j) = nwt(8);
    end
    
    resx = nanols(y,ones(size(y)));                         % calculate excess portfolio returns
    b_xret(j) = resx.beta*100;
    t_xret(j) = resx.tstat;
    nwt_xret(j) = meannwt(y);
end
%------------------- Long-Short Portfolio Regression ---------------------%
if n >= 2
    LSret  = pret(:,end) - pret(:,1);                           % long-short portfolio returns
    yls = LSret;

    resls = nanols(yls,x);
    R2_ls = resls.rsqr;
    alpha_ls = resls.beta(1)*100;
    talpha_ls = resls.tstat(1);
    residual_ls = res.resid;
    bstd_ls = res.bstd;
    nwt_ls = olsnwt(x,yls);
    nwtalpha_ls = nwt_ls(1);
    sharpe_ls = sqrt(12)*nanmean(yls)/nanstd(yls);
    info_ls = sqrt(12)*alpha_ls/(100*sqrt(resls.sige));

    b_factor1_ls = resls.beta(2);
    t_factor1_ls = resls.tstat(2);
    nwt_factor1_ls = nwt_ls(2);

    if method == 2 | method == 3 | method == 4 | method == 5 | method == 6 | method == 7;
        b_factor2_ls = resls.beta(3);
        t_factor2_ls = resls.tstat(3);
        nwt_factor2_ls = nwt_ls(3);
    end

    if method == 3 | method == 4 | method == 5 | method == 6 | method == 7;
        b_factor3_ls = resls.beta(4);
        t_factor3_ls = resls.tstat(4);
        nwt_factor3_ls = nwt_ls(4);
    end

    if method == 4 | method == 5 | method == 6 | method == 7;
        b_factor4_ls = resls.beta(5);
        t_factor4_ls = resls.tstat(5);
        nwt_factor4_ls = nwt_ls(5);
    end
    
    if method == 5 | method == 6 | method == 7;
        b_factor5_ls = resls.beta(6);
        t_factor5_ls = resls.tstat(6);
        nwt_factor5_ls = nwt_ls(6);
    end
    
    if method == 6 | method == 7;
        b_factor6_ls = resls.beta(7);
        t_factor6_ls = resls.tstat(7);
        nwt_factor6_ls = nwt_ls(7);
    end
    
    if method == 7;
        b_factor7_ls = resls.beta(8);
        t_factor7_ls = resls.tstat(8);
        nwt_factor7_ls = nwt_ls(8);
    end

    resx = nanols(yls,ones(size(yls)));
    b_xret_ls = resx.beta*100;                                  % average excess long-short portfolio return
    t_xret_ls = resx.tstat;
    nwt_xret_ls = meannwt(yls);
end
%---------------------------- Store Results ------------------------------%
if n >= 2
    a = [alpha alpha_ls];
    t = [talpha talpha_ls];
    nwt = [nwtalpha nwtalpha_ls];
    residual = [residual residual_ls];
    bstd = [bstd bstd_ls];
    sharpe = [sharpe sharpe_ls];
    info = [info info_ls];
    b_factor1 = [b_factor1 b_factor1_ls];
    t_factor1 = [t_factor1 t_factor1_ls];
    nwt_factor1 = [nwt_factor1 nwt_factor1_ls];

    if method == 2 | method == 3 | method == 4 | method == 5 | method == 6 | method == 7;
        b_factor2 = [b_factor2 b_factor2_ls];
        t_factor2 = [t_factor2 t_factor2_ls];
        nwt_factor2 = [nwt_factor2 nwt_factor2_ls];
    end;

    if method == 3 | method ==4 | method ==5 | method == 6 | method == 7;;
        b_factor3 = [b_factor3 b_factor3_ls];
        t_factor3 = [t_factor3 t_factor3_ls];
        nwt_factor3 = [nwt_factor3 nwt_factor3_ls];
    end

    if method ==4 | method ==5 | method == 6 | method == 7;;
        b_factor4 = [b_factor4 b_factor4_ls];
        t_factor4 = [t_factor4 t_factor4_ls];
        nwt_factor4 = [nwt_factor4 nwt_factor4_ls];
    end

    if  method == 5 | method == 6 | method == 7;;
        b_factor5 = [b_factor5 b_factor5_ls];
        t_factor5 = [t_factor5 t_factor5_ls];
        nwt_factor5 = [nwt_factor5 nwt_factor5_ls];
    end
    
    if  method == 6 | method == 7;;
        b_factor6 = [b_factor6 b_factor6_ls];
        t_factor6 = [t_factor6 t_factor6_ls];
        nwt_factor6 = [nwt_factor6 nwt_factor6_ls];
    end
    
    if  method == 7;;
        b_factor7 = [b_factor7 b_factor7_ls];
        t_factor7 = [t_factor7 t_factor7_ls];
        nwt_factor7 = [nwt_factor7 nwt_factor7_ls];
    end

    b_xret = [b_xret b_xret_ls];
    t_xret = [t_xret t_xret_ls];
    nwt_xret = [nwt_xret nwt_xret_ls];
    r = [R2 R2_ls];

    results.meth = '1 Factor';
    results.pret = pret;
    results.r2 = r;
    results.sharpe = sharpe;
    results.info = info;
    results.alpha = a;
    results.talpha = t;
    results.nwtalpha = nwt;
    results.resid = residual;
    results.bstd = bstd;
    results.xret = b_xret;
    results.txret = t_xret;
    results.nwtxret = nwt_xret;
    results.factor1 = b_factor1;
    results.tfactor1 = t_factor1;
    results.nwtfactor1 = nwt_factor1;

    if method == 2 | method == 3 | method == 4 | method == 5 | method == 6 | method == 7;
        results.meth = '2 Factors';
        results.factor2 = b_factor2;
        results.tfactor2 = t_factor2;
        results.nwtfactor2 = nwt_factor2;
    end

    if method == 3 | method == 4 | method == 5 | method == 6 | method == 7;
        results.meth = '3 Factors';
        results.factor3 = b_factor3;
        results.tfactor3 = t_factor3;
        results.nwtfactor3 = nwt_factor3;
    end

    if method == 4 | method == 5 | method == 6 | method == 7;
        results.meth = '4 Factors';
        results.factor4 = b_factor4;
        results.tfactor4 = t_factor4;
        results.nwtfactor4 = nwt_factor4;
    end

    if method == 5 | method == 6 | method == 7;
        results.meth = '5 Factors';
        results.factor5 = b_factor5;
        results.tfactor5 = t_factor5;
        results.nwtfactor5 = nwt_factor5;
    end
    
    if method == 6 | method == 7;
        results.meth = '6 Factors';
        results.factor6 = b_factor6;
        results.tfactor6 = t_factor6;
        results.nwtfactor6 = nwt_factor6;
    end
    
    if method == 7;
        results.meth = '7 Factors';
        results.factor7 = b_factor7;
        results.tfactor7 = t_factor7;
        results.nwtfactor7 = nwt_factor7;
    end
else
    a = alpha;
    t = talpha;
    nwt = nwtalpha;
    results.resid = residual;
    results.bstd = bstd;
    sharpe = sharpe;
    info = info;
    b_factor1 = b_factor1;
    t_factor1 = t_factor1;
    nwt_factor1 = nwt_factor1;

    if method == 2 | method == 3 | method ==4 | method ==5 | method == 6 | method == 7;
        b_factor2 = b_factor2;
        t_factor2 = t_factor2;
        nwt_factor2 = nwt_factor2;
    end;

    if method == 3 | method ==4 | method ==5 | method == 6 | method == 7;
        b_factor3 = b_factor3;
        t_factor3 = t_factor3;
        nwt_factor3 = nwt_factor3;
    end

    if method ==4 | method ==5 | method == 6 | method == 7;
        b_factor4 = b_factor4;
        t_factor4 = t_factor4;
        nwt_factor4 = nwt_factor4;
    end

    if  method == 5 | method == 6 | method == 7;
        b_factor5 = b_factor5;
        t_factor5 = t_factor5;
        nwt_factor5 = nwt_factor5;
    end
    
    if  method == 6 | method == 7;
        b_factor6 = b_factor6;
        t_factor6 = t_factor6;
        nwt_factor6 = nwt_factor6;
    end
    
    if  method == 7;
        b_factor7 = b_factor7;
        t_factor7 = t_factor7;
        nwt_factor7 = nwt_factor7;
    end

    b_xret = b_xret;
    t_xret = t_xret;
    nwt_xret = nwt_xret;
    r = R2;

    results.meth = '1 Factor';
    results.pret = pret;
    results.r2 = r;
    results.sharpe = sharpe;
    results.info = info;
    results.alpha = a;
    results.talpha = t;
    results.nwtalpha = nwt;
    results.resid = residual;
    results.bstd = bstd;
    results.xret = b_xret;
    results.txret = t_xret;
    results.nwtxret = nwt_xret;
    results.factor1 = b_factor1;
    results.tfactor1 = t_factor1;
    results.nwtfactor1 = nwt_factor1;

    if method == 2 | method == 3 | method == 4 | method == 5 | method == 6 | method == 7;
        results.meth = '2 Factors';
        results.factor2 = b_factor2;
        results.tfactor2 = t_factor2;
        results.nwtfactor2 = nwt_factor2;
    end

    if method == 3 | method == 4 | method == 5 | method == 6 | method == 7;
        results.meth = '3 Factors';
        results.factor3 = b_factor3;
        results.tfactor3 = t_factor3;
        results.nwtfactor3 = nwt_factor3;
    end

    if method == 4 | method == 5 | method == 6 | method == 7;
        results.meth = '4 Factors';
        results.factor4 = b_factor4;
        results.tfactor4 = t_factor4;
        results.nwtfactor4 = nwt_factor4;
    end

    if method == 5 | method == 6 | method == 7;
        results.meth = '5 Factors';
        results.factor5 = b_factor5;
        results.tfactor5 = t_factor5;
        results.nwtfactor5 = nwt_factor5;
    end
    
    if method == 6 | method == 7;
        results.meth = '6 Factors';
        results.factor6 = b_factor6;
        results.tfactor6 = t_factor6;
        results.nwtfactor6 = nwt_factor6;
    end
    
    if method == 7;
        results.meth = '7 Factors';
        results.factor7 = b_factor7;
        results.tfactor7 = t_factor7;
        results.nwtfactor7 = nwt_factor7;
    end
end
end

