function [ results ] = fmbregnwConst( y,x )
% the function is for running Fama MacBeth regression
% y and x represent dependent or independent variables respectively. The
% function differs with the function 'fmbregnw' is that this function
% reports constant term in the regression
warning off;
%------------------------pre-process and check data------------------------%
row = rows(y);

ind  = sum(~isnan(y),2)>0 & sum(~isnan(x),2)>0; % recognise the rows with values both in dependent or independent variables
y = y(ind,:);                                   % eliminate rows which have missing values in dependent or independent variables
x = x(ind,:);

[T,N] = size(y);    % this is to check dependent and independent variables 
Tx = rows(x);       % have same rows (length of dates)

if T ~= Tx 
    
    error('x and y do not have the same number of rows');

end

[Tx,Nx] = size(x);  % this is to check dependent and independent variables
k = Nx / N;         % have same columns (number of stocks)

if round(k)~=k 
    
    error('check the number of regressors, k is not an integer');

end
%--------------------periodic cross-section regressions--------------------%
beta = nan*zeros(T,k+1);    % T is the length of dates, N is number of stocks, and k is number of independent variables
for t = 1:T                 % transfer data into cross-section
    
    a = y(t,:)';
    
    b = x(t,:)';
   
    b = reshape(b,N,k);

    I = sum(isnan([a b]),2);
    
    a = a(I==0,:);
    
    b = b(I==0,:);
    
    if rows(a)>=0

        b = [ones(rows(b),1) b];            % the added column is for calculating constant
        
        res = nanols(a,b);                  % the ols function is downloaded from James LeSage
        
        beta(t,1:k+1) = [res.beta(1:end)]';
        
        rsqr(t) = res.rsqr;
    end
end
%-----------------average cross-section regression results-----------------%


beta(sum(~isnan(beta),2)~=k+1,:) = nan ;

i = ones(rows(beta),1);

for j= 1:cols(beta)
    
    res = nanols(beta(:,j),i);
    
    bhat(j) = res.beta;
    
    t(j) = olsnwt(i,beta(:,j));         % this is newey-west adjusted t-stat
end

rsqr(rsqr == 0) = [];

Rbar_squared = mean(rsqr);


%------------------------------store results-------------------------------%
B = nan*zeros(row,cols(beta)) ;

B(ind,:) = beta ;

results.beta = B;

results.bhat = bhat;

results.t = t;

results.rbarsquare = Rbar_squared;

in.rnames = ' ' ;

for i=1:k+1

    in.rnames = strvcat(in.rnames,['                    var ' int2str(i-1)]) ;

end

in.fmt=strvcat('%12.3f');
fprintf('---------------------------------------------------------------------- \n')
fprintf('           Results from Fama - Mac Beth regressions \n')
fprintf('---------------------------------------------------------------------- \n')
fprintf('Rbar-squared = %12.3f\n',Rbar_squared)
m = [bhat' t'];
in.cnames = strvcat('Coeff','t stat');

mprint(m,in)
fprintf('---------------------------------------------------------------------- \n')
warning on;
